package com.jelurida.ardor.contracts;

import nxt.addons.AbstractContract;
import nxt.addons.BlockContext;
import nxt.addons.ChainWrapper;
import nxt.addons.Contract;
import nxt.addons.DelegatedContext;
import nxt.addons.JO;
import nxt.http.callers.GetAccountPropertiesCall;
import nxt.http.callers.GetBlocksCall;
import nxt.http.callers.SendMoneyCall;
import nxt.http.responses.BlockResponse;
import nxt.util.Logger;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class ForgingReward extends AbstractContract {

    @Override
    public void processBlock(BlockContext context) {
        JO contractParams = getContractParams();
        int interval = contractParams.getInt("interval", 360);
        if (context.getBlock().getHeight() % interval != 0) {
            return;
        }
        boolean isRewardArdor = contractParams.getBoolean("rewardArdor", true);
        if (isRewardArdor) {
            sendReward(context, interval, null);
        }
        boolean isRewardNxt = contractParams.getBoolean("rewardNxt", true);
        String nxtNodeAddress = contractParams.getString("nxtNodeAddress", "http://localhost:7876/nxt");
        if (isRewardNxt) {
            URL nxtUrl;
            try {
                nxtUrl = new URL(nxtNodeAddress);
            } catch (MalformedURLException e) {
                throw new IllegalStateException(e);
            }
            sendReward(context, interval, nxtUrl);
        }
    }

    private void sendReward(BlockContext context, int interval, URL remoteUrl) {
        GetBlocksCall getBlocksCall = GetBlocksCall.create().firstIndex(0).lastIndex(interval - 1);
        if (remoteUrl != null) {
            getBlocksCall.remote(remoteUrl);
        }
        JO response = getBlocksCall.call();
        List<JO> blocks = response.getJoList("blocks");
        Map<String, Long> collect = blocks.stream().collect(Collectors.groupingBy(b -> BlockResponse.create(b).getGenerator(), Collectors.counting()));
        collect.keySet().removeIf(g -> {
            JO getAccountPropertiesResponse = GetAccountPropertiesCall.create().property("NoForgingReward").setter(g).recipient(g).call();
            return getAccountPropertiesResponse.getJoList("properties").size() > 0;
        });
        if (collect.size() == 0) {
            Logger.logInfoMessage("All forgers gave up on the reward");
        }
        Contract<Map<String, Long>, String> distributedRandomNumberGenerator = context.loadContract("DistributedRandomNumberGenerator");
        context.initRandom(0); // No secret data we can provide, need to make sure to set runner seed and not disclose it to block generators
        DelegatedContext delegatedContext = new DelegatedContext(context, distributedRandomNumberGenerator.getClass().getName());
        distributedRandomNumberGenerator.processInvocation(delegatedContext, collect);
        String selectedAccount = distributedRandomNumberGenerator.processInvocation(delegatedContext, collect);

        ChainWrapper chainWrapper = context.getChain("IGNIS");
        SendMoneyCall sendMoneyCall = SendMoneyCall.create(chainWrapper.getId()).recipient(selectedAccount).amountNQT(Math.multiplyExact(interval, chainWrapper.getOneCoin()));
        context.createTransaction(sendMoneyCall);
    }
}
